#!/usr/bin/perl -w
###############################################################################
##    Copyright (C) 2002 by Eric Gerbier
##    Bug reports to: gerbier@users.sourceforge.net
##    $Id: afick-tk.pl,v 1.19 2003/11/13 09:33:14 gerbier Exp $
##
##    This program is free software; you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation; either version 2 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
################################################################################
# this program just provide a graphical interface to afick
# just launch afick commands
################################################################################
use strict;
use Getopt::Long;    # arg analysis
use Tk;                                         # interface
require Tk::ROText;                             # to have read-only text

# global var
my $version = "0.4.0";
my $text;                                       # text widget for output display

# command arguments
my $configfile = '';
my $help       = '';
my $verbose    = '';
my $Version    = '';
my $Report_full_newdel;
my $Warn_missing_file;
my $Warn_dead_symlinks;

my @verbose_values     = qw(verbose_config verbose noverbose);
my @missing_values     = qw(missing_files_config missing_files nomissing_files);
my @dead_sym_values    = qw(dead_symlinks_config dead_symlinks nodead_symlinks);
my @full_newdel_values = qw(full_newdel_config full_newdel nofull_newdel);

# default file
my $default_conf;
if ( $^O =~ m/linux/ ) {
	$default_conf = "/etc/afick.conf";
}
elsif ( $^O =~ m/^MSWin/ ) {
	$default_conf = "xp.conf";
}
else {
	$default_conf = "afick.conf";
}

#####################################################################
# used to clear output screen before each command
sub clear_text() {
	$text->delete( '1.0', 'end' );
}
#####################################################################
# used to spawn afick commands
sub do_action($) {
	my $arg = shift (@_);

	$arg .= " -c \"$configfile\"" if ($configfile);

	# force values (overload values in config file)
	if ( $verbose eq $verbose_values[1] ) {
		$arg .= " --verbose";
	}
	elsif ( $verbose eq $verbose_values[2] ) {
		$arg .= " --noverbose";
	}

	if ( $Warn_missing_file eq $missing_values[1] ) {
		$arg .= " --missing_files";
	}
	elsif ( $Warn_missing_file eq $missing_values[2] ) {
		$arg .= " --nomissing_files";
	}

	if ( $Report_full_newdel eq $full_newdel_values[1] ) {
		$arg .= " --full_newdel";
	}
	elsif ( $Report_full_newdel eq $full_newdel_values[2] ) {
		$arg .= " --nofull_newdel";
	}

	if ( $Warn_dead_symlinks eq $dead_sym_values[1] ) {
		$arg .= " --dead_symlinks";
	}
	elsif ( $Warn_dead_symlinks eq $dead_sym_values[2] ) {
		$arg .= " --nodead_symlinks";
	}

	print "arg = $arg \n" if ( $verbose eq $verbose_values[1] );

	my $cursor = $text->cget('-cursor');

	$text->configure( -cursor => 'clock' );
	clear_text();
	$text->update();

	open( ACTION, "afick.pl $arg  2>&1 |" )
	  or die "can not execute afick.pl : $!\n";
	while (<ACTION>) {
		$text->insert( 'end', $_ );
	}
	close(ACTION);

	$text->see('end');

	# restore parameters
	$text->configure( -cursor => $cursor );
}

#####################################################################
# display help page
sub do_help($) {
	my $main = shift;

	my $top = $main->Toplevel;
	$top->Label( -text => "help" )->pack;
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin =
	  "this is a graphical interface to afick (another file integrity checker)
to monitor file system changes

top buttons :\n
init : to create the database\n
update : compare and update the database\n
compare : compare the files with the database\n
print : print database content\n
save  : save output screen to a local file\n
load  : display a saved outpout in outpout screen\n
help : this page\n
about : legal informations\n
exit : to quit this interface\n

you then have the afick output screen\n

then a line of radio-buttons for option, corresponding to command-line options :
verbose : produce more verbose output (for debugging purpose)
missing_files : warn about files in config file but missing
dead_links : warn about dead symlinks
full_newdel : produce a full report on new/deleted directories

then a  part for configuration\n
configuration file : to enter a configuration file name
select... : to select a configuration file
view/edit : to view or edit the selected configuration file

";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
# display about page
sub do_about($) {
	my $main = shift;

	my $top = $main->Toplevel;
	$top->Label( -text => "about" )->pack;
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )->pack();
	my $text = $top->Scrolled(
		'ROText',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

	my $baratin = "afick-tk version $version : a graphical interface to afick\n
url  : http://afick.sourceforge.net
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>
send remarks or bug reports to gerbier\@users.sourceforge.net

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

";

	$text->insert( 'end', $baratin );
	$text->see('1.0');
}
#####################################################################
# save output screen to a log file
sub do_save_log($) {
	my $main = shift;

	my $types = [ [ 'txt files', '.txt' ], [ 'All Files', '*', ], ];

	my @logs = $text->get( '1.0', 'end' );
	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) = gmtime(time);
	$year += 1900;
	$mon  += 1;
	my $date    = $year . "_" . "$mon" . "_" . $mday . "_" . $hour . "_" . $min;
	my $logname = "afick." . $date . ".txt";
	my $filename =
	  $main->getSaveFile( -initialfile => "$logname", -filetypes => $types );
	if ($filename) {
		print "log in $filename\n";
		open LOG, "> $filename" or warn "can not write on $filename\n";
		print LOG @logs;
		close LOG;
	}
}
#####################################################################
# save output screen to a log file
sub do_load_log($) {
	my $main = shift;

	my $types = [ [ 'txt files', '.txt' ], [ 'All Files', '*', ], ];

	my @logs = $text->get( '1.0', 'end' );
	my $filename = $main->getOpenFile( -filetypes => $types );
	if ($filename) {
		clear_text();

		open LOG, "< $filename" or warn "can not read $filename\n";
		while ( my $line = <LOG> ) {
			$text->insert( 'end', $line );
		}
		$text->see('1.0');
		close LOG;
	}
}
#####################################################################
# to select a config file
sub select_config($$) {
	my $FenetreP = shift;
	my $entree   = shift;

	my $types = [ [ 'config files', '.conf' ], [ 'All Files', '*', ], ];

	my $filename = $FenetreP->getOpenFile( -filetypes => $types );
	if ( defined $filename and $filename ne '' ) {
		$entree->delete( 0, 'end' );
		$entree->insert( 0, $filename );
		$entree->xview('end');
	}
}
#####################################################################
# to save a config file
sub save_config($$$) {
	my $main     = shift;
	my $text     = shift;
	my $top      = shift;
	my $types    = [ [ 'config files', '.conf' ], [ 'All Files', '*', ], ];
	my $initfile = $configfile;
	$initfile =~ s?/?\\?g;
	my $filename =
	  $main->getSaveFile( -initialfile => "$initfile", -filetypes => $types );

	my @conf = $text->get( '1.0', 'end' );
	open( CONF, "> $configfile" ) or die "can not open $configfile : $!";
	print CONF @conf;
	close CONF;
	$top->destroy();
}
#####################################################################
# open selected config file
sub ouvrir($) {
	my $main = shift;

	$configfile = $configfile || $default_conf;

	my $top = $main->Toplevel;
	$top->Label( -text => $configfile )->pack();
	my $text = $top->Scrolled(
		'Text',
		-scrollbars => 'e',
		-height     => 25,
		-width      => 128,
		-wrap       => 'word'
	)->pack( -side => "left", -expand => 1, -fill => 'both' );

#$top->Button( -text => "save" ,  -command => sub { save_config($main, $text, $top) } )->pack();
	my $SaveButton = $top->Button(
		-text    => "save",
		-command => sub { save_config( $main, $text, $top ) }
	);
	$SaveButton->pack();
	$top->Button( -text => "quit", -command => [ $top => 'destroy' ] )
	  ->pack( -after => $SaveButton );

	open( CONF, "< $configfile" ) or die "can not open $configfile : $!";
	while (<CONF>) {
		$text->insert( 'end', $_ );
	}
	close(CONF);
	$text->see('1.0');

}
#####################################################################
sub usage($) {

	my $version = shift (@_);
	print "\n";
	print "afick-tk ($version) : a graphical interface to afick\n";
	print "Usage: afick-tk [options]\n\n";
	print "Options:\n\n";
	print "--config_file|-c config\nname of config file to use\n\n";
	print
"--full_newdel|-f\nreport full  information on new and deleted directories\n\n";
	print "--help|-h\nthis help\n\n";
	print
"--missing_files|-m\nwarn about files declared in config files which does not exists\n\n";
	print "--dead_symlinks|-s\nwarn about dead symlinks\n\n";
	print "--verbose|-v\nverbose mode (for debugging purpose)\n\n";
	print "--version|-V\nprint program version\n\n";

	print

	  "#################################################################
This script is intended to provide a means for
detecting changes made to files, via a regular
comparison of MD5 hashes to an established baseline. 
#################################################################
Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
#################################################################\n";
}
#####################################################################
##                       main
#####################################################################

# options
Getopt::Long::Configure('no_ignore_case');
unless (
	GetOptions(
		"config_file|c=s"  => \$configfile,
		"full_newdel|f!"   => \$Report_full_newdel,
		"help|h"           => \$help,
		"missing_files|m!" => \$Warn_missing_file,
		"dead_symlinks|s!" => \$Warn_dead_symlinks,
		"verbose|v!"       => \$verbose,
		"version|V"        => \$Version

	)
  )
{
	usage($version);
	die "incorrect option\n";
}

if ($help) {
	usage($version);
	exit;
}

if ($Version) {
	print "afick-tk  version $version\n";
	exit;
}

# tk page
my $main = MainWindow->new( -title => "afick-tk $version" );

# frame 1 : boutons
my $frame1 = $main->Frame;
$frame1->pack;

$frame1->Button( -text => "init", -command => sub { do_action("-i") } )
  ->pack( -side => "left" );
$frame1->Button( -text => "update", -command => sub { do_action("-u") } )
  ->pack( -side => "left" );
$frame1->Button( -text => "compare", -command => sub { do_action("-k") } )
  ->pack( -side => "left" );
$frame1->Button( -text => "print", -command => sub { do_action("-p") } )
  ->pack( -side => "left" );

#$frame1->Button(
#	-text    => "clear",
#	-command => \&clear_text)->pack( -side => "left" );
$frame1->Button( -text => "save", -command => sub { do_save_log($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "load", -command => sub { do_load_log($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "help", -command => sub { do_help($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "about", -command => sub { do_about($main) } )
  ->pack( -side => "left" );
$frame1->Button( -text => "exit", -command => sub { exit } )
  ->pack( -side => "left" );

# frame 2  : configuration file
my $frame2 = $main->Frame;
$frame2->pack( -side => "bottom" );

$frame2->Label( -text => "configuration file", -anchor => 'e' )
  ->pack( -side => "left" );
my $entry =
  $frame2->Entry( -textvariable => \$configfile, -width => 30 )
  ->pack( -side => "left", -expand => 'yes' );
$frame2->Button(
	-text    => 'Select...',
	-command => sub { select_config( $main, $entry ) }
)->pack( -side => "left" );
$frame2->Button(
	-text    => 'View/Edit',
	-command => sub { ouvrir($main) }
)->pack( -side => "bottom" );

# frame 3 : verbose
my $frame3 = $main->Frame;
$frame3->pack( -side => "bottom" );

$frame3->Optionmenu(
	-options  => [@verbose_values],
	-variable => \$verbose
)->pack( -side => "left" );
$frame3->Optionmenu(
	-options  => [@missing_values],
	-variable => \$Warn_missing_file
)->pack( -side => "left" );
$frame3->Optionmenu(
	-options  => [@dead_sym_values],
	-variable => \$Warn_dead_symlinks
)->pack( -side => "left" );
$frame3->Optionmenu(
	-options  => [@full_newdel_values],
	-variable => \$Report_full_newdel
)->pack( -side => "left" );

# frame 4 : text vue
my $frame4 = $main->Frame;
$frame4->pack( -side => "bottom", -expand => 1, -fill => 'both' );

$text = $frame4->Scrolled(
	'ROText',
	-scrollbars => 'e',
	-height     => 25,
	-width      => 128,
	-wrap       => 'word'
)->pack( -side => "left", -expand => 1, -fill => 'both' );

MainLoop;

__END__

=head1 NAME

afick-tk : a graphical interface for afick (Another File Integrity Checker)

=head1 SYNOPSIS

=head1 OPTIONS

=head1 DESCRIPTION

=cut
